/* PYVMS__FILE_OPEN.C -- 26-DEC-1999 Uwe Zessin

04-AUG-2000 ZE. -- change 'struct PyFileObject' for upcoming 1.6/2.0 release

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  PYVMS__FILE_OPEN

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module PYVMS__FILE_OPEN "20000804"
#endif

#include "vmsdef.h"		/* includes "Python.h" */

#include "lib$routines.h"

/* ------------------------------------------------------------------------- */
/* some of this code is from BLTINMODULE.C + FILEOBJECT.C */

/* ------------------------------------------------------------------------- */
extern PyObject *pyvms_gr_error;		/* exception pyvms.error */

/* ------------------------------------------------------------------------- */
char pyvms_file_open__doc[] =
"file_object = pyvms.file_open (filename, [mode], [buffering], [,'RMS arguments'])\n\
\n\
Open a file.  The mode can be 'r', 'w' or 'a' for reading (default),\n\
writing or appending.  The file will be created if it doesn't exist\n\
when opened for writing or appending; it will be truncated when\n\
opened for writing.  Add a 'b' to the mode for binary files.\n\
Add a '+' to the mode to allow simultaneous reading and writing.\n\
If the buffering argument is given, 0 means unbuffered, 1 means line\n\
buffered, and larger numbers specify the buffer size.";

/* ---------------------------------------- */
/* @@ copy from FILEOBJECT.C */
typedef struct {
	PyObject_HEAD
	FILE *f_fp;
	PyObject *f_name;
	PyObject *f_mode;
	int (*f_close)(FILE *);
	int f_softspace; /* Flag used by 'print' command */
} PyFileObject;

/* ---------------------------------------- */
        /* PyFile_FromString */
PyObject * PYVMS_File_FromString
	(char *name
	,char *mode
	,unsigned long *al_arguments)
{
	extern int fclose (FILE *);
	PyFileObject *f;

	/* -------------------- */
	/* calls FILEOBJECT.C->PyFile_FromFile() */
	f = (PyFileObject *) PyFile_FromFile ((FILE *)NULL, name, mode, fclose);
	if (f == NULL)
	{
	    return NULL;
	}

	/* -------------------- */
	/* Py_BEGIN_ALLOW_THREADS */
	f->f_fp = (FILE *)lib$callg (al_arguments, fopen);
	/* Py_END_ALLOW_THREADS */

	/* ---------- */
	if (f->f_fp == NULL)
	{
#if 0
	    if (errno == EVMSERR)
	    {
		return PyVMS_ErrSetVal(pyvms_gr_error, 1, vaxc$errno); /* error */
	    }
	    else
	    {
		return PyErr_SetFromErrno(pyvms_gr_error);
	    }
#endif
	    PyErr_SetFromErrnoWithFilename(PyExc_IOError, name);
	    Py_DECREF(f);
	    return NULL;
	}

	/* -------------------- */
	return (PyObject *)f;
} /* PYVMS_File_FromString () */

/* ---------------------------------------- */

PyObject * pyvms_file_open (PyObject *self, PyObject *args)
{
	char *name;
	char *mode = "r";
	int bufsize = -1;
	PyObject *f;

	PyObject	* ar_filarg;
	unsigned long	* al_arguments;

	/* -------------------- */
	/* mode   = "r"; */
	bufsize   = -1;
	ar_filarg = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "s|siO:object_open",
	    &name, &mode, &bufsize, &ar_filarg))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 4: file attributes - tuple of strings */
	if (ar_filarg == Py_None)
	{
	    al_arguments = 0;		/* omitted */
	    f = PyFile_FromString (name, mode);
	}
	else
	{
	    unsigned long	  l_tuple_size;
	    unsigned long	  l_tuple_index;
	    unsigned long	* al_arguments_ptr;
	    PyObject		* ar_tuple_element;
	    unsigned long	  l_malloc_bytes;

	    if (!PyTuple_Check(ar_filarg))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: must be a tuple of strings");
		return NULL;
	    }

	    l_tuple_size = PyTuple_Size (ar_filarg);

	    /* allocate memory for argument array */
	    /* '+4', because the first element is the argument count and */
	    /*  then arguments 'file', 'flag' + 'mode' */
	    l_malloc_bytes = (l_tuple_size+4) * sizeof(al_arguments_ptr);
	    al_arguments = (unsigned long *)malloc (l_malloc_bytes);
	    if (al_arguments == NULL)
	    {
		return PyErr_NoMemory();
	    }

	    al_arguments_ptr = al_arguments;

	    /* store argument count */  /* 2= name,mode - arguments */
	    *al_arguments_ptr = 2 + l_tuple_size;
	    al_arguments_ptr++;
	    *al_arguments_ptr = (int)name;
	    al_arguments_ptr++;
	    *al_arguments_ptr = (int)mode;
	    al_arguments_ptr++;		/* next argument is RMS string */

	    /* Py_INCREF(ar_filarg) */
	    for (l_tuple_index = 0; l_tuple_index < l_tuple_size; l_tuple_index++)
	    {
		ar_tuple_element = PyTuple_GetItem(ar_filarg, l_tuple_index);
		if (ar_tuple_element == NULL)
		{
		    (void) free (al_arguments);
		    /* Py_DECREF(ar_filarg) */
		    return NULL;
		}
		if (!PyString_Check(ar_tuple_element))
		{
		    (void) free (al_arguments);
		    /* Py_DECREF(ar_filarg) */
		    return PyErr_Format(PyExc_TypeError,
			"argument 4: tuple-element:%d is not a string",
			l_tuple_index);
		}

		*al_arguments_ptr = (int)PyString_AS_STRING(ar_tuple_element);
		al_arguments_ptr++;
	    } /* loop over tuple */
	    /* ---------- */

	    f = PYVMS_File_FromString (name, mode, al_arguments);
	    (void) free (al_arguments);	/* release list */

	} /* else- (ar_filarg == Py_None) */

	/* -------------------- */
	if (f != NULL)
	{
	    PyFile_SetBufSize(f, bufsize);
	}

	/* -------------------- */
	return f;

} /* pyvms__file_open () */

/* ------------------------------------------------------------------------- */

/* EOF: PYVMS__FILE_OPEN.C */
