/* PYVMS__PYLIB_ROUTINES.C -- 22-JAN-2000 Uwe Zessin

$ @PYTHON_VMS:BLDRUN VMS PYVMS__PYLIB_ROUTINES

Special support for Python library modules.

22-JAN-2000 ZE. -- add LIB_GETPASS
*/

#ifdef __DECC
#pragma module PYVMS__PYLIB_ROUTINES "20000122"
#endif

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <dcdef.h>
#include <dvidef.h>
#include <iodef.h>
#include <lib$routines.h>
#include <ssdef.h>		/* SS$_name	 */
#include <starlet.h>		/* SYS$name, ... */
#include <stsdef.h>

/* ------------------------------------------------------------------------- */
extern PyObject *pyvms_gr_error;		/* exception pyvms.error */
/* ------------------------------------------------------------------------- */

char pyvms_pylib_getpass__doc[] =
"password = pyvms.pylib_getpass (fileno, prompt)\n\
Support routine for [.LIB]GETPASS.PY";

/* ------------------------------------------------------------------------- */

PyObject * pyvms_pylib_getpass (PyObject *self, PyObject *args)
{
#define S_PASSWORD 128	/* size of password buffer */

	unsigned long			  l_fileno;
	char				* at_prompt;
	unsigned long			  l_prompt_len;

	char				* at_buffer;
	char				* at_namebuf;

	unsigned short			  w_chan;
	unsigned short			  aw_iosb[4];
	unsigned long			  l_status;
	unsigned long			  l_status2;

	static char			  t_dummybuf [2];
	static char			  t_password [S_PASSWORD+1];

	char				* at_cr = "\r";

	struct dsc$descriptor_s		  r_io;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "is#",
	    &l_fileno,
	    &at_prompt, &l_prompt_len ))
	{
	    return NULL;
	}

	/* -------------------- */
	if (l_prompt_len > 65535)
	{
	    PyErr_SetString(PyExc_ValueError,
	       "argument 2: prompt - string size limited to 65535 characters");
	    return NULL;
	}

	/* -------------------- */
	at_buffer = malloc (512);
	if (at_buffer == NULL)
	{
	    PyErr_NoMemory();
	    return NULL;
        }

	/* -------------------- */
	/* convert |fileno| into OpenVMS-format filespec */
	/* '1' = OpenVMS format */
	at_namebuf = getname (l_fileno, at_buffer, 1);
	if (at_namebuf == NULL)
	{
	    (void) free (at_buffer);
	    return PyErr_SetFromErrno(pyvms_gr_error);
	}

	/* set up descriptor */
	r_io.dsc$w_length  = strlen (at_buffer);
	r_io.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_io.dsc$b_class   = DSC$K_CLASS_S;
	r_io.dsc$a_pointer = at_buffer;

	l_status = sys$assign(&r_io, &w_chan, 0, 0);

	if ((l_status & STS$M_SUCCESS) != 0)
	{
	    unsigned long	l_item_code = DVI$_DEVCLASS;
	    unsigned long	l_item_value;
	    unsigned long	l_dvi_status;

	    /* make sure IOs go to a terminal */
	    l_dvi_status = lib$getdvi
		(&l_item_code
		,&w_chan
		,0		/* device_name      */
		,&l_item_value
		,0		/* resultant_string */
		,0		/* resultant_length */
		);
	    if (l_dvi_status != SS$_NORMAL)
	    {
		(void) free (at_buffer);
		sys$dassgn (w_chan);
		return PyVMS_ErrSetVal(pyvms_gr_error, 1, l_status);
	    }
	    if (l_item_value != DC$_TERM)
	    {
		(void) free (at_buffer);
		sys$dassgn (w_chan);
		return PyVMS_ErrSetVal(pyvms_gr_error, 1, SS$_IVDEVNAM);
	    }

	    /* -------------------- */
	    /* read password with prompt */
            l_status = sys$qiow
			(0
			,w_chan
			,IO$_READPROMPT | IO$M_NOECHO
			,&aw_iosb
			,0
			,0
			,t_password
			,sizeof(t_password) - 1
			,0
			,0
			,at_prompt
			,l_prompt_len
			);

	    if ((l_status & STS$M_SUCCESS) != 0)
	    {
                l_status = (((int)aw_iosb[0]) & 0xffff);
                if ((l_status & STS$M_SUCCESS) != 0)
		{
		    t_password[aw_iosb[1]] = '\0';
                }

		/* write a <CR> via prompt to put cursor in column 1 */
		l_status2 = sys$qiow		/* ignore status */
			(0			/* efn           */
			,w_chan			/* I/O channel   */
			,IO$_READPROMPT | IO$M_NOECHO | IO$M_TIMED /* func */
			,0			/* ignore IOSB */
			,0			/* astadr */
			,0			/* astprm */
			,t_dummybuf		/* P1: input buffer */
			,sizeof(t_dummybuf) - 1 /* P2: input length */
			,0			/* P3: timeout */
			,0
			,at_cr			/* P5: prompt buffer */
			,1			/* P6: prompt length */
			);

            }
            sys$dassgn (w_chan);
	}

	/* -------------------- */
	if ((l_status & STS$M_SUCCESS) == 0)
	{
	    (void) free (at_buffer);
	    lib$stop (l_status);
            return PyVMS_ErrSetVal(pyvms_gr_error, 1, l_status);
	}

	/* -------------------- */
	/* return Py_BuildValue ("s", at_namebuf); */
	(void) free (at_buffer);
	return Py_BuildValue ("s", t_password);

} /* pyvms_pylib_getpass () */

/* ------------------------------------------------------------------------- */

/* EOF: PYVMS__PYLIB_ROUTINES.C */
