/* VMS_LBR_GET_INDEX.C -- 18-OCT-1999 Uwe Zessin
   Python interface to LBR$GET_INDEX
*/

#ifdef __DECC
#pragma module VMS_LBR_GET_INDEX "19991221"
#endif

/* @@@ routine_name only uses descriptor for parameter 1 --
   library with ASCII keys */

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>

/* ------------------------------------------------------------------------- */
extern PyObject *vms_lbr_gr_error;		/* exception vms_lbr.error */
/* ------------------------------------------------------------------------- */

/* ---------------------------------------- */
/* some DEC C versions are missing LBR$ROUTINES.H */
#ifdef __DECC
#ifndef __DECC_VER	/* don't know if this is defined in all versions ... */
#define __DECC_VER 0
#endif /* __DECC_VER */
#if (__DECC_VER < 60090001) /* @@ might need to change for older version */
#ifndef __unknown_params
#define __unknown_params ...
#endif
#pragma __nostandard	  /* non-ANSI-Standard feature */
unsigned long lbr$get_index(__unknown_params);
#pragma __standard
#else
#include <lbr$routines.h>
#endif /* __DECC_VER < 60090001 */
#endif /* __DECC */

/* ------------------------------------------------------------------------- */

static PyObject * vms_lbr__ar_routine_name; /* Python callback routine */

/* ---------------------------------------- */
static unsigned long vms_lbr__routine_name
    (struct dsc$descriptor_s * ar_key_name, unsigned long * al_rfa)
{
	PyObject * ar_py_params;	/* P1 */
	PyObject * ar_status;

	ar_py_params = Py_BuildValue ("(s#(ii))",
	    ar_key_name->dsc$a_pointer,
	    (unsigned int)ar_key_name->dsc$w_length,
	    *al_rfa, *(al_rfa + 1)
	    );
	if (ar_py_params == NULL)
	{
	    (void) PyErr_Print();
	    return 0;			/* assume error has been set up */
	}
	ar_status = PyObject_CallObject
		   (vms_lbr__ar_routine_name, ar_py_params);
	Py_DECREF(ar_py_params);	/* drop this object */
	if (ar_status == NULL)
	{
	    (void) PyErr_Print();
	    return 0;			/* assume error has been set up */
	}
	return 1;	/* bit0 - success */
} /* vms_lbr__routine_name() */

/* ------------------------------------------------------------------------- */

char vms_lbr_get_index__doc[] =
"vms_lbr.get_index (library_index, index_number, routine_name, [match_desc])\n\
Call a Routine for Selected Index Keys.";

/* ---------------------------------------- */

PyObject *
vms_lbr_get_index (PyObject *self, PyObject *args)
{
	unsigned long		  l_library_index;
	unsigned long		  l_index_number;

	PyObject		* ar_routine_name;
	unsigned long		* al_routine_name;

	char			* at_match_desc;
	struct dsc$descriptor_s   r_match_desc;
	struct dsc$descriptor_s * ar_match_desc;
	unsigned long		  l_match_desc_len;

	unsigned long		  l_status;

	/* -------------------- */
	ar_routine_name = Py_None;
	at_match_desc   = NULL;     l_match_desc_len = 0;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "llO|s#",
	    &l_library_index,
	    &l_index_number,
	    &ar_routine_name,
	    &at_match_desc,    &l_match_desc_len ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: library_index */

	/* -------------------- */
	/* argument 2: index_number */

	/* -------------------- */
	/* argument 3: routine_name */

	/* argument must be a Python function object */
	if (!PyFunction_Check (ar_routine_name))
	{
	    PyErr_SetString(PyExc_TypeError,
		"argument 3: routine_name - must be function or None");
	    return NULL;
	}
	vms_lbr__ar_routine_name = ar_routine_name; /* store object */
	al_routine_name = (unsigned long *)&vms_lbr__routine_name;

	/* -------------------- */
	Py_XINCREF(vms_lbr__ar_routine_name);

	/* -------------------- */
	/* argument 4: match_desc */

	if (at_match_desc == NULL)
	{
	    ar_match_desc = 0;		/* omitted */
	}
	else
	{
            if (l_match_desc_len > 65535)
            {
		PyErr_SetString(PyExc_ValueError,
	    "argument 4: match_desc - string size limited to 65535 characters");
		return NULL;
	    }
	    r_match_desc.dsc$w_length  = l_match_desc_len;
	    r_match_desc.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_match_desc.dsc$b_class   = DSC$K_CLASS_S;
	    r_match_desc.dsc$a_pointer = at_match_desc;
	    ar_match_desc = &r_match_desc;
	}

	/* -------------------- */
	l_status = lbr$get_index
		(&l_library_index
		,&l_index_number
		,al_routine_name
		,ar_match_desc
		);

	/* -------------------- */
	/* release Python function objects */
	Py_XDECREF(vms_lbr__ar_routine_name);

	/* -------------------- */
	if (l_status == 2523137) /* LBR$_NORMAL */
	{
	    Py_INCREF (Py_None);
	    return Py_None;
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_lbr_gr_error, 1, l_status);
} /* vms_lbr_get_index() */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_LBR_GET_INDEX.C */
