/* VMS_LBR_OPEN.C -- 25-SEP-1999 Uwe Zessin
   Python interface to LBR$OPEN
*/

#ifdef __DECC
#pragma module VMS_LBR_OPEN "19991221"
#endif

#include "python.h"
#include "vmsdef.h"
#include "vmsobj_nam.h"

#include <descrip.h>
#include <lib$routines.h>
#include <nam.h>
#include <ssdef.h>

/* ------------------------------------------------------------------------- */
extern PyObject *vms_lbr_gr_error;		/* exception vms_lbr.error */
/* ------------------------------------------------------------------------- */

/* ---------------------------------------- */
/* some DEC C versions are missing LBR$ROUTINES.H */
#ifdef __DECC
#ifndef __DECC_VER	/* don't know if this is defined in all versions ... */
#define __DECC_VER 0
#endif /* __DECC_VER */
#if (__DECC_VER < 60090001) /* @@ might need to change for older version */
#ifndef __unknown_params
#define __unknown_params ...
#endif
#pragma __nostandard	  /* non-ANSI-Standard feature */
unsigned long lbr$open(__unknown_params);
#pragma __standard
#else
#include <lbr$routines.h>
#endif /* __DECC_VER < 60090001 */
#endif /* __DECC */

/* ------------------------------------------------------------------------- */

char vms_lbr_open__doc[] =
"rns = vms_lbr.open \
(library_index, [fns], [create_options], [dns], [rlfna],\
 func, [type], [namblk])\n\
Open or Create a Library.";

/* ---------------------------------------- */

PyObject *
vms_lbr_open (PyObject *self, PyObject *args)
{
	unsigned long		  l_library_index;

	char			* at_fns;
	struct dsc$descriptor_s   r_fns;
	struct dsc$descriptor_s * ar_fns;
	unsigned long		  l_fns_len;

	PyObject		* ar_create_options;
	unsigned long		  l_tuple_size;
	unsigned long		  l_tuple_index;
	unsigned long		  l_tuple_element;
	PyObject		* ar_tuple_element;
	unsigned long		  l_create_options[20];
	unsigned long		* al_create_options;

	char			* at_dns;
	struct dsc$descriptor_s   r_dns;
	struct dsc$descriptor_s * ar_dns;
	unsigned long		  l_dns_len;

	vmsobj_nam		* ar_rlfna;
	struct NAM		* ar_vms_nam;

	struct dsc$descriptor_s   r_rns;
	unsigned long		  l_rnslen;

	unsigned long		  l_status_lbr;
	unsigned long		  l_status_free;

	PyObject		* ar_return;

	/* -------------------- */
	at_fns            = NULL;    l_fns_len = 0;
	ar_create_options = Py_None;
	at_dns            = NULL;    l_dns_len = 0;
	ar_rlfna          = (vmsobj_nam *)Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "l|z#Oz#O",
	    &l_library_index,
	    &at_fns,            &l_fns_len,
	    &ar_create_options,
	    &at_dns,            &l_dns_len,
	    &ar_rlfna ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: l_library_index */

	/* -------------------- */
	/* argument 2: fns - library file specification */
	if (at_fns == NULL)
	{
	    ar_fns = 0;		/* omitted */
	}
	else
	{
	    if (l_fns_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
		  "argument 2: fns - string size limited to 65535 characters");
		return NULL;
	    }
	    r_fns.dsc$w_length  = l_fns_len;
	    r_fns.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_fns.dsc$b_class   = DSC$K_CLASS_S;
	    r_fns.dsc$a_pointer = at_fns;
	    ar_fns = &r_fns;
	}

	/* -------------------- */
	/* argument 3: create_options */
	if (ar_create_options == Py_None)
	{
	    al_create_options = 0;	/* omitted */
	}
	else
	{
	    if (!PyTuple_Check(ar_create_options))
	    {
		PyErr_SetString(PyExc_TypeError,
		"argument 3: create_options - must be a tuple of 20 integers");
		return NULL;
	    }

	    l_tuple_size = PyTuple_Size(ar_create_options);
	    if (l_tuple_size != 20)
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: create_options - must be a tuple of 20 integers");
		return NULL;
	    }

	    for (l_tuple_index = 0; l_tuple_index < 20; l_tuple_index++)
	    {
		ar_tuple_element = PyTuple_GetItem(ar_create_options, l_tuple_index);
		if (ar_tuple_element == NULL)
		{
		    return NULL;
		}
		if (!PyInt_Check(ar_tuple_element))
		{
		    return PyErr_Format(PyExc_TypeError,
		   "argument 3: create_options - tuple-element:%d is not an integer",
		    l_tuple_index);
		}

		l_tuple_element = PyInt_AsLong(ar_tuple_element);
		if (PyErr_Occurred())
		{
		    return NULL;
		}

		l_create_options[l_tuple_index] = l_tuple_element;
	    }
	    al_create_options = &l_create_options[0];
	}

	/* -------------------- */
	/* argument 4: dns - default file specification */
	if (at_dns == NULL)
	{
	    ar_dns = 0;		/* omitted */
	}
	else
	{
	    if (l_dns_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
		  "argument 4: dns - string size limited to 65535 characters");
		return NULL;
	    }
	    r_dns.dsc$w_length  = l_dns_len;
	    r_dns.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_dns.dsc$b_class   = DSC$K_CLASS_S;
	    r_dns.dsc$a_pointer = at_dns;
	    ar_dns = &r_dns;
	}

	/* -------------------- */
	/* argument 5: rlfna */
	if (ar_rlfna == (vmsobj_nam *)Py_None)
	{
	    ar_vms_nam = 0;
	}
	else
	{
	    if (! vmsobj_nam_Check(ar_rlfna))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: rlfna - must be vmsobj_nam object or None");
		return NULL;
	    }
	    ar_vms_nam = ar_rlfna->ab_nam;
	}

	/* -------------------- */
	/* set up string descriptor */
	/*  let LBR$OPEN allocate the memory */
	r_rns.dsc$w_length  = 0;
	r_rns.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_rns.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_rns.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_lbr = lbr$open
		(&l_library_index
		,ar_fns
		,al_create_options
		,ar_dns
		,ar_vms_nam
		,&r_rns
		,&l_rnslen
		);

	/* -------------------- */
	/* note: need to return status value */

	/* -------------------- */
#ifndef LBR$_NORMAL
#define LBR$_NORMAL 2523137
#endif
	if (l_status_lbr == LBR$_NORMAL)
	{
	    PyObject	* ar_strobj;

	    ar_strobj = Py_BuildValue
		("s#", (l_rnslen == 0) ? /* empty string? */
		(void*)&r_rns	 :    /* non-NULL address */
		(void*)r_rns.dsc$a_pointer
		,l_rnslen);
	    if (ar_strobj != NULL)
	    {
		ar_return = Py_BuildValue ("lO",
				l_status_lbr, ar_strobj);
		Py_DECREF(ar_strobj);
	    }
	    else
	    {
		ar_return = NULL;
	    }
	}
	else
	{
	    ar_return = NULL;
	}

	/* -------------------- */
	/* deallocate memory of dynamic string descriptor */
	l_status_free = lib$sfree1_dd (&r_rns);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_lbr_open: LIB$SFREE1_DD() failed");
	    (void) PyErr_Print();

	    if (l_status_lbr == SS$_NORMAL)
	    {
		l_status_lbr = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
	    }
	}

	/* -------------------- */
	if (l_status_lbr == LBR$_NORMAL)
	{
	    return ar_return;
	}

	Py_XDECREF(ar_return);

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_lbr_gr_error, 1, l_status_lbr);
} /* vms_lbr_open() */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_LBR_OPEN.C */
