/* VMS_SMG_CREATE_MENU.C -- 02-APR-2000 Uwe Zessin
   Python interface to SMG$CREATE_MENU

This file contains (modified) code from:
  Bill Nixon, Univ. of Pittsburgh, Computing and Information Systems
  Written  6:17 pm  Dec 19, 1988 by wpn@cisunx.UUCP in comp.os.vms
  Subject: "Sample VAX C program using new SMG$"

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CREATE_MENU

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CREATE_MENU "20000402"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_create_menu__doc[] =
"None = vms_smg.create_menu (display_id, choices, [menu_type], [flags],\
 [row], [rendition_set], [rendition_complement])\n\
Fill the Virtual Display with a Menu.";

/* ------------------------------------------------------------------------- */

/* Structure Description for 'static array of char_string' */
/* (Modification of the Array Descriptor as defined in "descrip.h" */

struct  dsc_descriptor_sac
{
	unsigned short	dsc$w_length;	/* length of each string in array */
	unsigned char	dsc$b_dtype;	/* data type code = DSC$K_DTYPE_T */
	unsigned char	dsc$b_class;	/* desc class code= DSC$K_CLASS_A */
	char	      * dsc$a_pointer;	/* address of char array[][]      */
	char		dsc$b_scale;	/* set to 0			  */
	char		dsc$b_digits;	/* set to 0			  */
	struct
	{
		unsigned                 : 4;	/* reservered, must be zero */
		unsigned dsc$v_fl_redim  : 1;	/* set to zero		    */
		unsigned dsc$v_fl_column : 1;	/* set to zero		    */
		unsigned dsc$v_fl_coeff  : 1;	/* set to one, mult block   */
		unsigned dsc$v_fl_bounds : 1;	/* set to one, bounds block */
	} dsc$b_aflags;
	unsigned char	dsc$b_dimct;	/* MUST set to one                */
	unsigned long	dsc$l_arsize;	/* Total size of char array[][]	  */
	char		*dsc$a_a0;	/* address of char array[][]	  */
	long		dsc$l_m;	/* num of strings in array        */
	long		dsc$l_l;	/* set to zero, lower bound       */
	long		dsc$l_u;	/* num of strings-1, upper bound  */
};

/* -------------------- */

PyObject *
vms_smg_create_menu (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	PyObject		* ar_py_choices;
	PyObject		* ar_py_choiceobj;  /* a single menu item */

	unsigned long		  l_choices_count;  /* number of choice  */
	unsigned long		  l_choice_idx;     /* index into choice */
	unsigned long		  l_choices_maxlen; /* longes choice     */

	unsigned long		  l_malloc_bytes;
	char			* at_choices_str;   /* whole array */
	char			* at_choice_ptr;    /* individual item addr */

	/* Descriptor to pass as the 'choice' argument in SMG$CREATE_MENU */
	struct dsc_descriptor_sac r_sac;

	PyObject		* ar_menu_type;
	unsigned long		  l_menu_type;
	unsigned long		* al_menu_type;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	PyObject		* ar_row;
	long			  l_row;
	long			* al_row;

	PyObject		* ar_rendition_set;
	unsigned long		  l_rendition_set;
	unsigned long		* al_rendition_set;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	unsigned long		  l_status;

	/* -------------------- */
	ar_menu_type            = Py_None;
	ar_flags                = Py_None;
	ar_row                  = Py_None;
	ar_rendition_set        = Py_None;
	ar_rendition_complement = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "iO|OOOOO"
	    ,&l_display_id
	    ,&ar_py_choices
	    ,&ar_menu_type
	    ,&ar_flags
	    ,&ar_row
	    ,&ar_rendition_set
	    ,&ar_rendition_complement
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: choices */
	/*  this one is checked later on to ease handling of */
	/*  dynamically allocated memory */

	/* -------------------- */
	/* argument 3: [menu_type] */
	if (ar_menu_type == Py_None)
	{
	    al_menu_type = 0;		/* omitted */
	    l_menu_type  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_menu_type))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: menu_type - must be integer or None");
		return NULL;
	    }
	    l_menu_type = PyInt_AsLong(ar_menu_type);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_menu_type = &l_menu_type;
	}

	/* -------------------- */
	/* argument 4: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 4: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	/* argument 5: [row] */
	if (ar_row == Py_None)
	{
	    al_row = 0;		/* omitted */
	    l_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: row - must be integer or None");
		return NULL;
	    }
	    l_row = PyInt_AsLong(ar_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_row = &l_row;
	}

	/* -------------------- */
	/* argument 6: [rendition_set] */
	if (ar_rendition_set == Py_None)
	{
	    al_rendition_set = 0;		/* omitted */
	    l_rendition_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 6: rendition_set - must be integer or None");
		return NULL;
	    }
	    l_rendition_set = PyInt_AsLong(ar_rendition_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_set = &l_rendition_set;
	}

	/* -------------------- */
	/* argument 7: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 7: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	/* argument 2: choices */
	if (!PyTuple_Check(ar_py_choices))
	{
	    PyErr_SetString(PyExc_TypeError,
		"argument 2: choices - must be a tuple of strings");
	    return NULL;
	}

	/* ---------- */
	l_choices_count = PyTuple_Size(ar_py_choices);

	/* ---------- */
	/* loop over all tuple items to check if: */
	/*  - each one is a string  */
	/*  - its length is < 65536 */
	/*  - find out which one is the longest */

	l_choices_maxlen = 0;
	for (l_choice_idx = 0; l_choice_idx < l_choices_count; l_choice_idx++)
	{
	    unsigned long l_strsiz;

	    ar_py_choiceobj = PyTuple_GetItem(ar_py_choices,l_choice_idx);
	    if (ar_py_choiceobj == NULL)
	    {
		return NULL;
	    }
	    if (!PyString_Check(ar_py_choiceobj))
	    {
		(void) PyErr_Format(PyExc_TypeError,
		    "argument 2: choices - choice:%d must be a string",
		    l_choice_idx);
		return NULL;
	    }
	    l_strsiz = PyString_GET_SIZE(ar_py_choiceobj);
	    if (l_strsiz > 65535)
	    {
		(void) PyErr_Format(PyExc_ValueError,
		    "argument 2: choices - choice:%d string too long",
		    l_choice_idx);
		return NULL;
	    }
	    if (l_strsiz > l_choices_maxlen)
	    {
		l_choices_maxlen = l_strsiz;
	    }
	}

	/* ---------- */
	/* allocate a large chunk of memory to hold all strings */
	l_malloc_bytes = l_choices_count * l_choices_maxlen;
	at_choices_str = malloc (l_malloc_bytes);
	if (at_choices_str == NULL)
	{
	    return PyErr_NoMemory();
	}
	/* Set the allocated memory to 0. */
	(void) memset (at_choices_str, 0, l_malloc_bytes);

	/* ---------- */
	/* again loop over tuple - this time copy strings */
	at_choice_ptr = at_choices_str;
	for (l_choice_idx = 0; l_choice_idx < l_choices_count; l_choice_idx++)
	{
	    unsigned long l_strsiz;
	    char * at_straddr;

	    ar_py_choiceobj = PyTuple_GetItem(ar_py_choices,l_choice_idx);
	    if (ar_py_choiceobj == NULL)
	    {
		(void) free(at_choices_str);
		return NULL;
	    }
	    at_straddr = PyString_AS_STRING(ar_py_choiceobj);
	    l_strsiz   = PyString_GET_SIZE(ar_py_choiceobj);
	    (void) memcpy (at_choice_ptr, at_straddr, l_strsiz);

	    /* next address in array */
	    at_choice_ptr += l_choices_maxlen;
	}

	/* Define the 'choice' argument descriptor */
	r_sac.dsc$w_length  = l_choices_maxlen;	/* Length of each element */
	r_sac.dsc$b_dtype   = DSC$K_DTYPE_T;	/* String Type            */
	r_sac.dsc$b_class   = DSC$K_CLASS_A;	/* Array Class            */
	r_sac.dsc$a_pointer = at_choices_str;	/* Point to C char array  */
	r_sac.dsc$b_scale   = 0;		/* Set as indicated above */
	r_sac.dsc$b_digits  = 0;
	r_sac.dsc$b_aflags.dsc$v_fl_coeff  = 1;
	r_sac.dsc$b_aflags.dsc$v_fl_bounds = 1;
	r_sac.dsc$b_dimct  = 1;
	r_sac.dsc$l_arsize = l_choices_maxlen;	/* Size of the C char array  */
	r_sac.dsc$a_a0 = at_choices_str;	/* Point once again to array */
	r_sac.dsc$l_m  = l_choices_count;	/* Num of elements/choices   */
	r_sac.dsc$l_l  = 0;			/* Lower index 		     */
	r_sac.dsc$l_u  = l_choices_count-1;	/* Upper index 		     */

	/* -------------------- */
	l_status = smg$create_menu
		(&l_display_id
		,&r_sac
		,al_menu_type
		,al_flags
		,al_row
		,al_rendition_set
		,al_rendition_complement
		);

	/* -------------------- */
	/* release dynamically allocated memory */
	(void) free(at_choices_str);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_create_menu () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CREATE_MENU.C */
