/* VMS_SMG_CREATE_PASTEBOARD.C -- 28-FEB-2000 Uwe Zessin
   Python interface to SMG$CREATE_PASTEBOARD

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CREATE_PASTEBOARD

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CREATE_PASTEBOARD "20000228"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name  */
#include <smg$routines.h>	/* SMG$name  */
#include <smgmsg.h>		/* SMG$_name */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_create_pasteboard__doc[] =
"status, pasteboard_id, number_of_pasteboard_rows,\
 number_of_pasteboard_columns, type_of_terminal, device_name\
 = vms_smg.create_pasteboard ([output_device], [flags])\n\
Create a Pasteboard.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_create_pasteboard (PyObject *self, PyObject *args)
{
	char			* at_output_device;
	struct dsc$descriptor_s   r_output_device;
	struct dsc$descriptor_s * ar_output_device;
	unsigned long		  l_output_device_len;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	unsigned long		  l_pasteboard_id;
	long			  l_number_of_pasteboard_rows;
	long			  l_number_of_pasteboard_columns;
	unsigned long		  l_type_of_terminal;

	struct dsc$descriptor_s   r_device_name;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	PyObject		* ar_return;

	/* -------------------- */
	at_output_device = NULL; l_output_device_len = 0;
	ar_flags = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "|z#O",
	    &at_output_device, &l_output_device_len,
	    &ar_flags))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: output_device */
	if (at_output_device == NULL)
	{
	    ar_output_device = 0;	/* omitted */
	}
	else
	{
	    if (l_output_device_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	"argument 1: output_device - string size limited to 65535 characters");
		return NULL;
	    }
	    r_output_device.dsc$w_length  = l_output_device_len;
	    r_output_device.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_output_device.dsc$b_class   = DSC$K_CLASS_S;
	    r_output_device.dsc$a_pointer = at_output_device;
	    ar_output_device = &r_output_device;
	}

	/* -------------------- */
	/* argument 2: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	l_pasteboard_id                = 0;
	l_number_of_pasteboard_rows    = 0;
	l_number_of_pasteboard_columns = 0;
	l_type_of_terminal             = 0;

	/* set up string descriptor */
	/*  let SMG$CREATE_PASTEBOARD allocate the memory */
	r_device_name.dsc$w_length  = 0;
	r_device_name.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_device_name.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_device_name.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$create_pasteboard
		(&l_pasteboard_id
		,ar_output_device
		,&l_number_of_pasteboard_rows
		,&l_number_of_pasteboard_columns
		,al_flags
		,&l_type_of_terminal
		,&r_device_name
		);

	/* -------------------- */
	if ((l_status_smg == SS$_NORMAL) || (l_status_smg == SMG$_PASALREXI))
	{
	    PyObject	* ar_strobj;

	    ar_strobj = Py_BuildValue
		("s#"
		,(r_device_name.dsc$w_length == 0)	/* empty string? */
		  ? (void*)&r_device_name		/* non-NULL address */
		  : (void*)r_device_name.dsc$a_pointer
		,(unsigned int)r_device_name.dsc$w_length
		);
	    if (ar_strobj != NULL)
	    {
		ar_return = Py_BuildValue ("iiiiiO"
			,l_status_smg
			,l_pasteboard_id
			,l_number_of_pasteboard_rows
			,l_number_of_pasteboard_columns
			,l_type_of_terminal
			,ar_strobj);
		Py_DECREF(ar_strobj);
	    }
	    else
	    {
		ar_return = NULL;
	    }
	}
	else
	{
	    ar_return = NULL;
	}

	/* -------------------- */
	/* deallocate memory of dynamic string descriptor */
	l_status_free = lib$sfree1_dd (&r_device_name);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_smg_create_pasteboard: LIB$SFREE1_DD() failed");
	    (void) PyErr_Print();

	    if (l_status_smg == SS$_NORMAL)
	    {
		l_status_smg = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
	    }
	}

	/* -------------------- */
	if ((l_status_smg == SS$_NORMAL) || (l_status_smg == SMG$_PASALREXI))
	{
	    return ar_return;
	}

	Py_XDECREF(ar_return);

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_smg);
} /* vms_smg_create_pasteboard () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CREATE_PASTEBOARD.C */
