/* VMS_SMG_LOAD_KEY_DEFS.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$LOAD_KEY_DEFS

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_LOAD_KEY_DEFS

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_LOAD_KEY_DEFS "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_load_key_defs__doc[] =
"None = vms_smg.load_key_defs\
 (key_table_id, [filespec], [default_filespec], [flags])\n\
Load Key Definitions.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_load_key_defs (PyObject *self, PyObject *args)
{
	unsigned long		  l_key_table_id;

	char			* at_filespec;
	struct dsc$descriptor_s   r_filespec;
	unsigned long		  l_filespec_len;

	char			* at_default_filespec;
	struct dsc$descriptor_s   r_default_filespec;
	struct dsc$descriptor_s * ar_default_filespec;
	unsigned long		  l_default_filespec_len;

	PyObject		* ar_flags;
	long			  l_flags;
	long			* al_flags;

	unsigned long		  l_status;

	/* -------------------- */
	at_filespec         = NULL;	l_filespec_len = 0;
	at_default_filespec = NULL;	l_default_filespec_len = 0;
	ar_flags            = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "is#|z#O"
	    ,&l_key_table_id
	    ,&at_filespec         ,&l_filespec_len
	    ,&at_default_filespec ,&l_default_filespec_len
	    ,&ar_flags
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: key_table_id */

	/* -------------------- */
	/* argument 2: filespec */
	if (l_filespec_len > 65535)
	{
	    PyErr_SetString(PyExc_ValueError,
	     "argument 2: filespec - string size limited to 65535 characters");
	    return NULL;
	}
	r_filespec.dsc$w_length  = l_filespec_len;
	r_filespec.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_filespec.dsc$b_class   = DSC$K_CLASS_S;
	r_filespec.dsc$a_pointer = at_filespec;

	/* -------------------- */
	/* argument 3: [default_filespec} */
	if (at_default_filespec == NULL)
	{
	    ar_default_filespec = 0;	/* omitted */
	}
	else
	{
	    if (l_default_filespec_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
     "argument 3: default_filespec - string size limited to 65535 characters");
		return NULL;
	    }
	    r_default_filespec.dsc$w_length  = l_default_filespec_len;
	    r_default_filespec.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_default_filespec.dsc$b_class   = DSC$K_CLASS_S;
	    r_default_filespec.dsc$a_pointer = at_default_filespec;
	    ar_default_filespec = &r_default_filespec;
	}

	/* -------------------- */
	/* argument 4: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	l_status = smg$load_key_defs
		(&l_key_table_id
		,&r_filespec
		,ar_default_filespec
		,al_flags
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_load_key_defs () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_LOAD_KEY_DEFS.C */
