/* VMS_SMG_READ_COMPOSED_LINE.C -- 25-MAR-2000 Uwe Zessin
   Python interface to SMG$READ_COMPOSED_LINE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_READ_COMPOSED_LINE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_READ_COMPOSED_LINE "20000325"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name */
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_read_composed_line__doc[] =
"status, resultant_string = vms_smg.read_composed_line\
 (keyboard_id, [key_table_id], [prompt_string],\
 [display_id], [flags], [initial_string], [timeout],\
 [rendition_set], [rendition_complement], [terminator_code])\n\
Read Composed Line.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_read_composed_line (PyObject *self, PyObject *args)
{
	unsigned long		  l_keyboard_id;

	PyObject		* ar_key_table_id;
	unsigned long		  l_key_table_id;
	unsigned long		* al_key_table_id;

	char			* at_prompt_string;
	struct dsc$descriptor_s   r_prompt_string;
	struct dsc$descriptor_s * ar_prompt_string;
	unsigned long		  l_prompt_string_len;

	PyObject		* ar_display_id;
	unsigned long		  l_display_id;
	unsigned long		* al_display_id;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	char			* at_initial_string;
	struct dsc$descriptor_s   r_initial_string;
	struct dsc$descriptor_s * ar_initial_string;
	unsigned long		  l_initial_string_len;

	PyObject		* ar_timeout;
	long			  l_timeout;
	long			* al_timeout;

	PyObject		* ar_rendition_set;
	unsigned long		  l_rendition_set;
	unsigned long		* al_rendition_set;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	PyObject		* ar_terminator_code;
	unsigned long		  l_terminator_code;
	unsigned short		  w_terminator_code;
	unsigned short		* aw_terminator_code;


	struct dsc$descriptor_s   r_resultant_string;
	unsigned short		  w_resultant_length;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	/* -------------------- */
	ar_key_table_id         = Py_None;
	at_prompt_string        = NULL;    l_prompt_string_len = 0;
	ar_display_id           = Py_None;
	ar_flags                = Py_None;
	at_initial_string       = NULL;    l_initial_string_len = 0;
	ar_timeout              = Py_None;
	ar_rendition_set        = Py_None;
	ar_rendition_complement = Py_None;
	ar_terminator_code      = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|Oz#OOz#OOOO"
	    ,&l_keyboard_id
	    ,&ar_key_table_id
	    ,&at_prompt_string       ,&l_prompt_string_len
	    ,&ar_display_id
	    ,&ar_flags
	    ,&at_initial_string      ,&l_initial_string_len
	    ,&ar_timeout
	    ,&ar_rendition_set
	    ,&ar_rendition_complement
	    ,&ar_terminator_code
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [key_table_id] */
	if (ar_key_table_id == Py_None)
	{
	    al_key_table_id = 0;		/* omitted */
	    l_key_table_id  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_key_table_id))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: key_table_id - must be integer or None");
		return NULL;
	    }
	    l_key_table_id = PyInt_AsLong(ar_key_table_id);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_key_table_id = &l_key_table_id;
	}

	/* -------------------- */
	/* argument 3: [prompt_string} */
	if (at_prompt_string == NULL)
	{
	    ar_prompt_string = 0;	/* omitted */
	}
	else
	{
	    if (l_prompt_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	"argument 3: prompt_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_prompt_string.dsc$w_length  = l_prompt_string_len;
	    r_prompt_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_prompt_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_prompt_string.dsc$a_pointer = at_prompt_string;
	    ar_prompt_string = &r_prompt_string;
	}

	/* -------------------- */
	/* argument 4: [display_id] */
	if (ar_display_id == Py_None)
	{
	    al_display_id = 0;		/* omitted */
	    l_display_id  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_display_id))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: display_id - must be integer or None");
		return NULL;
	    }
	    l_display_id = PyInt_AsLong(ar_display_id);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_display_id = &l_display_id;
	}

	/* -------------------- */
	/* argument 5: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	/* argument 6: [initial_string} */
	if (at_initial_string == NULL)
	{
	    ar_initial_string = 0;	/* omitted */
	}
	else
	{
	    if (l_initial_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
       "argument 6: initial_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_initial_string.dsc$w_length  = l_initial_string_len;
	    r_initial_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_initial_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_initial_string.dsc$a_pointer = at_initial_string;
	    ar_initial_string = &r_initial_string;
	}

	/* -------------------- */
	/* argument 7: [timeout] */
	if (ar_timeout == Py_None)
	{
	    al_timeout = 0;		/* omitted */
	    l_timeout  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_timeout))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 7: timeout - must be integer or None");
		return NULL;
	    }
	    l_timeout = PyInt_AsLong(ar_timeout);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_timeout = &l_timeout;
	}

	/* -------------------- */
	/* argument 8: [rendition_set] */
	if (ar_rendition_set == Py_None)
	{
	    al_rendition_set = 0;		/* omitted */
	    l_rendition_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 8: rendition_set - must be integer or None");
		return NULL;
	    }
	    l_rendition_set = PyInt_AsLong(ar_rendition_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_set = &l_rendition_set;
	}

	/* -------------------- */
	/* argument 9: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 9: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	/* argument 10: [terminator_code] */
	if (ar_terminator_code == Py_None)
	{
	    aw_terminator_code = 0;		/* omitted */
	    l_terminator_code  = 0;
	    w_terminator_code  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_terminator_code))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: terminator_code - must be integer or None");
		return NULL;
	    }
	    l_terminator_code = PyInt_AsLong(ar_terminator_code);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if (l_terminator_code > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 10: terminator_code - must be 0..65535");
		return NULL;
	    }
	    w_terminator_code = l_terminator_code;
	    aw_terminator_code = &w_terminator_code;
	}

	/* -------------------- */
	w_resultant_length = 0;
	/* set up string descriptor */
	/*  let SMG$READ_COMPOSED_LINE allocate the memory */
	r_resultant_string.dsc$w_length  = 0;
	r_resultant_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_resultant_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_resultant_string.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$read_composed_line
		(&l_keyboard_id
		,al_key_table_id
		,&r_resultant_string
		,ar_prompt_string
		,&w_resultant_length
		,al_display_id
		,al_flags
		,ar_initial_string
		,al_timeout
		,al_rendition_set
		,al_rendition_complement
		,aw_terminator_code
		);

	/* -------------------- */
	{
	    PyObject	* ar_return;

	    ar_return = Py_BuildValue
		("is#"
		,l_status_smg
		,(r_resultant_string.dsc$w_length == 0)	/* empty string? */
		  ? (void*)&r_resultant_string		/* non-NULL address */
		  : (void*)r_resultant_string.dsc$a_pointer
		,(unsigned int)r_resultant_string.dsc$w_length
		);

	    /* -------------------- */
	    /* deallocate memory of dynamic string descriptor */
	    l_status_free = lib$sfree1_dd (&r_resultant_string);
	    if (l_status_free != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_SystemError,
		    "vms_smg_read_composed_line: LIB$SFREE1_DD() failed");
		(void) PyErr_Print();

		Py_XDECREF(ar_return);

		/* -------------------- */
		/* error */
		return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_free);
	    }

	    /* -------------------- */
	    return ar_return;
	}

} /* vms_smg_read_composed_line () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_READ_COMPOSED_LINE.C */
