/* VMS_SMG_REPLACE_INPUT_LINE.C -- 26-MAR-2000 Uwe Zessin
   Python interface to SMG$REPLACE_INPUT_LINE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_REPLACE_INPUT_LINE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_REPLACE_INPUT_LINE "20000326"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_replace_input_line__doc[] =
"None = vms_smg.replace_input_line (keyboard_id, [replace_string],\
 [line_count], [flags])\n\
Replace Input Line.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_replace_input_line (PyObject *self, PyObject *args)
{
	unsigned long		  l_keyboard_id;

	char			* at_replace_string;
	struct dsc$descriptor_s   r_replace_string;
	struct dsc$descriptor_s * ar_replace_string;
	unsigned long		  l_replace_string_len;

	PyObject		* ar_line_count;
	unsigned long		  l_line_count;
	unsigned char		  b_line_count;
	unsigned char		* ab_line_count;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	unsigned long		  l_status;

	/* -------------------- */
	at_replace_string = NULL;    l_replace_string_len = 0;
	ar_line_count     = Py_None;
	ar_flags          = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|z#OO"
	    ,&l_keyboard_id
	    ,&at_replace_string ,&l_replace_string_len
	    ,&ar_line_count
	    ,&ar_flags
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: keyboard_id */

	/* -------------------- */
	/* argument 2: [replace_string] */
	if (at_replace_string == NULL)
	{
	    ar_replace_string = 0;	/* omitted */
	}
	else
	{
	    if (l_replace_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
       "argument 2: replace_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_replace_string.dsc$w_length  = l_replace_string_len;
	    r_replace_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_replace_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_replace_string.dsc$a_pointer = at_replace_string;
	    ar_replace_string = &r_replace_string;
	}

	/* -------------------- */
	/* argument 3: [line_count] */
	if (ar_line_count == Py_None)
	{
	    ab_line_count = 0;		/* omitted */
	    b_line_count  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_line_count))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: line_count - must be integer or None");
		return NULL;
	    }
	    l_line_count = PyInt_AsLong(ar_line_count);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if (l_line_count > 255)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 3: line_count - must be 0..255");
		return NULL;
	    }
	    b_line_count  = l_line_count;
	    ab_line_count = &b_line_count;
	}

	/* -------------------- */
	/* argument 4: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	l_status = smg$replace_input_line
		(&l_keyboard_id
		,ar_replace_string
		,ab_line_count
		,al_flags
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_replace_input_line () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_REPLACE_INPUT_LINE.C */
