/* VMS_SMG_SELECT_FROM_MENU.C -- 02-APR-2000 Uwe Zessin
   Python interface to SMG$SELECT_FROM_MENU

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_SELECT_FROM_MENU

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_SELECT_FROM_MENU "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name */
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_select_from_menu__doc[] =
"status, selected_choice_number, selected_choice_string =\
 vms_smg.select_from_menu\
 (keyboard_id, display_id, [default_choice_number], [flags], [help_library],\
 [timeout], [terminator_code], [rendition_set], [rendition_complement])\n\
Make a Selection from the Menu.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_select_from_menu (PyObject *self, PyObject *args)
{
	unsigned long		  l_keyboard_id;
	unsigned long		  l_display_id;

	PyObject		* ar_default_choice_number;
	long			  l_default_choice_number;
	unsigned short		  w_default_choice_number;
	unsigned short		* aw_default_choice_number;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	char			* at_help_library;
	struct dsc$descriptor_s   r_help_library;
	struct dsc$descriptor_s * ar_help_library;
	unsigned long		  l_help_library_len;

	PyObject		* ar_timeout;
	long			  l_timeout;
	long			* al_timeout;

	PyObject		* ar_terminator_code;
	unsigned long		  l_terminator_code;
	unsigned short		  w_terminator_code;
	unsigned short		* aw_terminator_code;

	PyObject		* ar_rendition_set;
	unsigned long		  l_rendition_set;
	unsigned long		* al_rendition_set;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	unsigned short		  w_selected_choice_number;
	struct dsc$descriptor_s   r_selected_choice_string;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	PyObject		* ar_return;

	/* -------------------- */
	ar_default_choice_number = Py_None;
	ar_flags                 = Py_None;
	at_help_library          = NULL;    l_help_library_len = 0;
	ar_timeout               = Py_None;
	ar_terminator_code       = Py_None;
	ar_rendition_set         = Py_None;
	ar_rendition_complement  = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "ii|OOz#OOOO"
	    ,&l_keyboard_id
	    ,&l_display_id
	    ,&ar_default_choice_number
	    ,&ar_flags
	    ,&at_help_library         ,&l_help_library_len
	    ,&ar_timeout
	    ,&ar_terminator_code
	    ,&ar_rendition_set
	    ,&ar_rendition_complement
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: keyboard_id */

	/* -------------------- */
	/* argument 2: display_id */

	/* -------------------- */
	/* argument 3: [default_choice_number] */
	if (ar_default_choice_number == Py_None)
	{
	    aw_default_choice_number = 0;		/* omitted */
	    l_default_choice_number  = 0;
	    w_default_choice_number  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_default_choice_number))
	    {
		PyErr_SetString(PyExc_TypeError,
		"argument 3: default_choice_number - must be integer or None");
		return NULL;
	    }
	    l_default_choice_number = PyInt_AsLong(ar_default_choice_number);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if (l_default_choice_number > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 3: default_choice_number - must be 0..65535");
	    }
	    w_default_choice_number  = l_default_choice_number;
	    aw_default_choice_number = &w_default_choice_number;
	}

	/* -------------------- */
	/* argument 4: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	/* argument 5: [help_library] */
	if (at_help_library == NULL)
	{
	    ar_help_library = 0;	/* omitted */
	}
	else
	{
	    if (l_help_library_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	 "argument 5: help_library - string size limited to 65535 characters");
		return NULL;
	    }
	    r_help_library.dsc$w_length  = l_help_library_len;
	    r_help_library.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_help_library.dsc$b_class   = DSC$K_CLASS_S;
	    r_help_library.dsc$a_pointer = at_help_library;
	    ar_help_library = &r_help_library;
	}

	/* -------------------- */
	/* argument 6: [timeout] */
	if (ar_timeout == Py_None)
	{
	    al_timeout = 0;		/* omitted */
	    l_timeout  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_timeout))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 6: timeout - must be integer or None");
		return NULL;
	    }
	    l_timeout = PyInt_AsLong(ar_timeout);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_timeout = &l_timeout;
	}

	/* -------------------- */
	/* argument 7: [terminator_code] */
	if (ar_terminator_code == Py_None)
	{
	    aw_terminator_code = 0;		/* omitted */
	    l_terminator_code  = 0;
	    w_terminator_code  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_terminator_code))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 7: terminator_code - must be integer or None");
		return NULL;
	    }
	    l_terminator_code = PyInt_AsLong(ar_terminator_code);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if (l_terminator_code > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 7: terminator_code - must be 0..65535");
	    }
	    w_terminator_code  = l_terminator_code;
	    aw_terminator_code = &w_terminator_code;
	}

	/* -------------------- */
	/* argument 8: [rendition_set] */
	if (ar_rendition_set == Py_None)
	{
	    al_rendition_set = 0;		/* omitted */
	    l_rendition_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 8: rendition_set - must be integer or None");
		return NULL;
	    }
	    l_rendition_set = PyInt_AsLong(ar_rendition_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_set = &l_rendition_set;
	}

	/* -------------------- */
	/* argument 9: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 9: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	w_selected_choice_number = 0;

	/* set up string descriptor */
	/*  let SMG$CREATE_PASTEBOARD allocate the memory */
	r_selected_choice_string.dsc$w_length  = 0;
	r_selected_choice_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_selected_choice_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_selected_choice_string.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$select_from_menu
		(&l_keyboard_id
		,&l_display_id
		,&w_selected_choice_number
		,aw_default_choice_number
		,al_flags
		,ar_help_library
		,al_timeout
		,aw_terminator_code
		,&w_selected_choice_number
		,al_rendition_set
		,al_rendition_complement
		);

	/* -------------------- */
	{
	    PyObject	* ar_strobj;

	    ar_strobj = Py_BuildValue
		("s#"
		,(r_selected_choice_string.dsc$w_length == 0)	/* empty string? */
		  ? (void*)&r_selected_choice_string		/* non-NULL address */
		  : (void*)r_selected_choice_string.dsc$a_pointer
		,(unsigned int)r_selected_choice_string.dsc$w_length
		);
	    if (ar_strobj != NULL)
	    {
		ar_return = Py_BuildValue ("iiO"
			,l_status_smg
			,(unsigned int)w_selected_choice_number
			,ar_strobj);
		Py_DECREF(ar_strobj);
	    }
	    else
	    {
		ar_return = NULL;
	    }
	}

	/* -------------------- */
	/* deallocate memory of dynamic string descriptor */
	l_status_free = lib$sfree1_dd (&r_selected_choice_string);
	if (l_status_free != SS$_NORMAL)
	{
	    PyErr_SetString(PyExc_SystemError,
		"vms_smg_select_from_menu: LIB$SFREE1_DD() failed");
	    (void) PyErr_Print();

	    if (l_status_smg == SS$_NORMAL)
	    {
		l_status_smg = l_status_free; /* fail anyway */
	        Py_XDECREF(ar_return);
	    }
	}

	/* -------------------- */
	return ar_return;
} /* vms_smg_select_from_menu () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_SELECT_FROM_MENU.C */
