/* VMS_SMG_SET_DEFAULT_STATE.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$SET_DEFAULT_STATE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_SET_DEFAULT_STATE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_SET_DEFAULT_STATE "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_set_default_state__doc[] =
"None = vms_smg.set_default_state (key_table_id, [new_state], [old_state])\n\
Set Default State.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_set_default_state (PyObject *self, PyObject *args)
{
	unsigned long		  l_key_table_id;

	char			* at_new_state;
	struct dsc$descriptor_s   r_new_state;
	struct dsc$descriptor_s * ar_new_state;
	unsigned long		  l_new_state_len;

	char			* at_old_state;
	struct dsc$descriptor_s   r_old_state;
	struct dsc$descriptor_s * ar_old_state;
	unsigned long		  l_old_state_len;

	unsigned long		  l_status;

	/* -------------------- */
	at_new_state = NULL; l_new_state_len = 0;
	at_old_state = NULL; l_old_state_len = 0;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|z#z#"
	    ,&l_key_table_id
	    ,&at_new_state   ,&l_new_state_len
	    ,&at_old_state   ,&l_old_state_len
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: key_table_id */

	/* -------------------- */
	/* argument 2: [new_state} */
	if (at_new_state == NULL)
	{
	    ar_new_state = 0;	/* omitted */
	}
	else
	{
	    if (l_new_state_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	    "argument 2: new_state - string size limited to 65535 characters");
		return NULL;
	    }
	    r_new_state.dsc$w_length  = l_new_state_len;
	    r_new_state.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_new_state.dsc$b_class   = DSC$K_CLASS_S;
	    r_new_state.dsc$a_pointer = at_new_state;
	    ar_new_state = &r_new_state;
	}

	/* -------------------- */
	/* argument 3: [old_state] */
	if (at_old_state == NULL)
	{
	    ar_old_state = 0;	/* omitted */
	}
	else
	{
	    if (l_old_state_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
	    "argument 2: old_state - string size limited to 65535 characters");
		return NULL;
	    }
	    r_old_state.dsc$w_length  = l_old_state_len;
	    r_old_state.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_old_state.dsc$b_class   = DSC$K_CLASS_S;
	    r_old_state.dsc$a_pointer = at_old_state;
	    ar_old_state = &r_old_state;
	}

	/* -------------------- */
	l_status = smg$set_default_state
		(&l_key_table_id
		,ar_new_state
		,ar_old_state
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_set_default_state () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_SET_DEFAULT_STATE.C */
