/* VMS_SYS_NUMUTC.C -- 16-JAN-2000 Uwe Zessin
   Python interface to SYS$NUMUTC()

12-DEC-1997 ZE. -- created in VMS_SYS.C
16-JAN-2000 ZE. -- extract into VMS_SYS_NUMUTC.C for V5.5-2 exclusion
*/

#ifdef __DECC
#pragma module VMS_SYS_NUMUTC "20000116"
#endif

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <ssdef.h>		/* SS$_name	 */
#include <starlet.h>		/* SYS$name, ... */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_sys_gr_error;	/* in VMS_SYS.C */
/* ------------------------------------------------------------------------- */

/* input a Python long integer, output a binary octaword */
extern long vms__cvt_pylong2octa (PyObject * r_longint, long * o_octaword);

/* ------------------------------------------------------------------------- */

char vms_sys_numutc__doc[] =
"timbuf = vms_sys.numtim ([utcadr])\n\
Convert UTC Time to Numeric Components.";

PyObject *
vms_sys_numutc (PyObject *self, PyObject *args)
{
	PyObject	* ar_utcadr;
	long		  o_utcadr[4];		/* utc_date_time */
	long		* ao_utcadr;

	struct	numutc_xr_timbuf {
		unsigned short int	w_year_since_0;
		unsigned short int	w_month_of_year;
		unsigned short int	w_day_of_month;
		unsigned short int	w_hour_of_day;
		unsigned short int	w_minute_of_hour;
		unsigned short int	w_second_of_minute;
		unsigned short int	w_hundredths_of_second;
		short int		w_inacc_days;	/* [1] */
		short int		w_inacc_hours;
		short int		w_inacc_minutes;
		short int		w_inacc_seconds;
		short int		w_inacc_hundredths_of_second;
		short int		w_tdf_in_minutes;
	} numutc_r_timbuf;

	/* [1] - signed variable, */
	/* because a negative value indicates infinite inaccuracy */

	unsigned long	  l_status;

	/* -------------------- */
	ar_utcadr = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "|O", &ar_utcadr))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: utcadr */
	if (ar_utcadr == Py_None)
	{
	    ao_utcadr = 0;		/* omitted */
	}
	else
	{
	    if (!PyLong_Check(ar_utcadr))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: utcadr - must be long integer");
		return NULL;
	    }
	    /* convert Python long integer to binary octaword */
	    l_status = vms__cvt_pylong2octa (ar_utcadr, &o_utcadr[0]);
	    if (l_status != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_ValueError,
		   "argument 1: utcadr - conversion from long integer failed");
		return NULL;
	    }
	    ao_utcadr = &o_utcadr[0];
	}

	/* -------------------- */
	l_status = sys$numutc
		(&numutc_r_timbuf	/* timbuf   */
		,ao_utcadr		/* [utcadr] */
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    /* return tuple to Python */
	    return Py_BuildValue
		("(iiiiiiiiiiiii)"
		,(unsigned int)numutc_r_timbuf.w_year_since_0
		,(unsigned int)numutc_r_timbuf.w_month_of_year
		,(unsigned int)numutc_r_timbuf.w_day_of_month
		,(unsigned int)numutc_r_timbuf.w_hour_of_day
		,(unsigned int)numutc_r_timbuf.w_minute_of_hour
		,(unsigned int)numutc_r_timbuf.w_second_of_minute
		,(unsigned int)numutc_r_timbuf.w_hundredths_of_second
/* [1] */	,(int)numutc_r_timbuf.w_inacc_days
		,(int)numutc_r_timbuf.w_inacc_hours
		,(int)numutc_r_timbuf.w_inacc_minutes
		,(int)numutc_r_timbuf.w_inacc_seconds
		,(int)numutc_r_timbuf.w_inacc_hundredths_of_second
		,(int)numutc_r_timbuf.w_tdf_in_minutes
		);
	}
/* [1] - signed variable, */
/* because a negative value indicates infinite inaccuracy */

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_sys_gr_error, 1, l_status);
} /* vms_sys_numutc () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SYS_NUMUTC.C */
